/*
 * ukui-system-appwidget's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangyan <wangyan@kylinos.cn>
 *
 */

import QtQuick 2.0
import QtQml 2.12
import QtQuick.Layouts 1.12
import org.ukui.appwidget 1.0
import QtGraphicalEffects 1.0
//import QtQuick.Controls 2.0
AppWidget {
    id:appwidget
    appname: "time"
    visible: true
    width: 448*scalefactor
    height: 162*scalefactor
    anchors.centerIn: parent
    property double scalefactor: 1.00
    property string timeformat: "12";
    property string dateformat: "cn";
    property string hour;
    property string minute;
    property string week;
    property string amorpm;
    property  bool v: false
    property string themename;
    property bool qmleffect: false
    property bool qmlenable: true

    Timer {
        id: gettime;
        interval: 100;
        running: true;
        repeat: true;
        triggeredOnStart: true
        onTriggered: {
            updateTimeInfo();
            updateWeakInfo();
            updateDateInfo();
        }
    }

    Rectangle {
        id :twelverec
        color: "transparent"
//        radius: 15
        anchors.fill: parent
        anchors.left: parent.left
        anchors.verticalCenter: parent.verticalCenter
        visible: !v
        Rectangle {
            id :time1rec
            color: "transparent"
            anchors.verticalCenter: parent.verticalCenter
            width: 244*appwidget.scalefactor
            height: 112*appwidget.scalefactor
            Text {
                id: time1
                anchors.fill:parent
                verticalAlignment: TextInput.AlignVCenter
                color: "#FAFAFA"
                font.pixelSize:112*appwidget.scalefactor
                font.letterSpacing: 0
                font.family: "STxihei"

                layer.enabled: appwidget.qmleffect
                layer.effect: DropShadow {
                    verticalOffset: 1
                    color: "#40262626"
                    samples: 4
                }
            }
        }
        Rectangle {
            //间隔
            id :spacing1
            width: 16*appwidget.scalefactor
            height: 112*appwidget.scalefactor
            anchors.left: time1rec.right
            anchors.verticalCenter: parent.verticalCenter
            color: "transparent"
        }
        Rectangle {
            id :daterec
            color: "transparent"
//            border.color: "pink"
            width: 137*appwidget.scalefactor
            height: 96*appwidget.scalefactor
            anchors.verticalCenter: parent.verticalCenter
            Layout.fillWidth: true
            anchors.left: spacing1.right

            ColumnLayout{
                width: 137*appwidget.scalefactor
                height: 96*appwidget.scalefactor
                anchors.centerIn: parent
                Text {
                    id: ampm
                    color: "#FAFAFA";
                    font.pixelSize:40*appwidget.scalefactor;
                    font.letterSpacing: 0
                    font.family: "STxihei"
                    verticalAlignment: TextInput.AlignVCenter

                    layer.enabled: appwidget.qmleffect
                    layer.effect: DropShadow {
                        verticalOffset: 1
                        color: "#40262626"
                        samples: 4
                    }
                }
                RowLayout {
                    Text {
                        id: week1
                        color: "#FAFAFA";
                        font.pixelSize: 28*appwidget.scalefactor;
                        font.letterSpacing: 0
                        font.family: "STxihei"
                        verticalAlignment: TextInput.AlignVCenter

                        layer.enabled: appwidget.qmleffect
                        layer.effect: DropShadow {
                            verticalOffset: 1
                            color: "#40262626"
                            samples: 4
                        }
                    }
                    Text {
                        id: date1
                        color: "#FAFAFA";
                        font.pixelSize: 28*appwidget.scalefactor;
                        font.letterSpacing: 0
                        font.family: "STxihei"
                        verticalAlignment: TextInput.AlignVCenter

                        layer.enabled: appwidget.qmleffect
                        layer.effect: DropShadow {
                            verticalOffset: 1
                            color: "#40262626"
                            samples: 4
                        }
                    }
                }
            }
        }
    }


    Rectangle{
        id :twentyfoursystem
        color: "transparent"
//        radius: 15
        anchors.fill: parent
        anchors.left: parent.left
        anchors.verticalCenter: parent.verticalCenter
        visible: v
        Rectangle {
            id :time2rec
            color: "transparent"
//            border.color: "pink"
            anchors.verticalCenter: parent.verticalCenter
            width: 294*appwidget.scalefactor
            height: 112*appwidget.scalefactor
            Text {
                id: time2
                anchors.fill:parent
                verticalAlignment: TextInput.AlignVCenter
                color: "#FAFAFA"
                font.pixelSize:112*appwidget.scalefactor
                font.letterSpacing:0
                font.family: "STxihei"

                layer.enabled: appwidget.qmleffect
                layer.effect: DropShadow {
                    verticalOffset: 1
                    color: "#40262626"
                    samples: 4
                }
            }
        }
        Rectangle {
            //间隔
            id :spacing2
            width: 16*appwidget.scalefactor
            height: 112*appwidget.scalefactor
            anchors.left: time2rec.right
            anchors.verticalCenter: parent.verticalCenter
            color: "transparent"
        }
        Rectangle {
            color: "transparent"
            width: 87*appwidget.scalefactor
            height: 112*appwidget.scalefactor
            Layout.fillWidth: true
            anchors.verticalCenter: parent.verticalCenter
            anchors.left: spacing2.right

            ColumnLayout{
                width: 87*appwidget.scalefactor
                height: 112*appwidget.scalefactor
                anchors.centerIn: parent
                Layout.fillWidth: true
                Text {
                    id: week2
                    color: "#FAFAFA";
                    font.pixelSize:40*appwidget.scalefactor;
                    font.letterSpacing: 0
                    font.family: "STxihei"
                    verticalAlignment: TextInput.AlignVCenter

                    layer.enabled: appwidget.qmleffect
                    layer.effect: DropShadow {
                        verticalOffset: 1
                        color: "#40262626"
                        samples: 4
                    }
                }
                Text {
                    id: date2
                    color: "#FAFAFA";
                    font.pixelSize:40*appwidget.scalefactor;
                    font.letterSpacing: 0
                    font.family: "STxihei"
                    verticalAlignment: TextInput.AlignVCenter

                    layer.enabled: appwidget.qmleffect
                    layer.effect: DropShadow {
                        verticalOffset: 1
                        color: "#40262626"
                        samples: 4
                    }
                }
            }
        }
    }

    Component.onCompleted: {

        initScalefactor()
        gettime.start();
        userStatusChangedSignal.connect(onUserStatusChanged)
        console.log("qml 接收到数据更新信号");

        appwidget.creatGsetting("org.ukui.control-center.panel.plugins");
        appwidget.gsettingUpdate.connect(appwidget.onGsettingUpdate);
        timeformat = appwidget.getGsettingValue("org.ukui.control-center.panel.plugins","hoursystem");
        dateformat = appwidget.getGsettingValue("org.ukui.control-center.panel.plugins","date");
        if(timeformat == "24") {
            v = true
        }else {
            v = false
        }
        console.log("当前gsetting为", timeformat, dateformat);
    }
    onWidthChanged: {
        console.log("qml 窗口宽度改变，重新计算缩放比");
        initScalefactor()
    }
    onHeightChanged: {
        console.log("qml 窗口高度改变，重新计算缩放比");
        initScalefactor()
    }
    function onGsettingUpdate(key, value)
    {
        if(key === "hoursystem")
        {
            timeformat = value
            if(timeformat == "24") {
                v = true
            }else {
                v = false
            }
        }
        if(key === "date")
        {
            dateformat = value
        }
    }
    function initScalefactor()
    {
        //按宽计算缩放比例,计算在这个比例系数下的小插件宽高，如果高超出父类的高，则更换比例系数的计算方式
        //按高计算缩放比例
        //console.log("父类窗口大小", appwidget.parent.width,appwidget.parent.height);
        if(appwidget.parent.width == 0 || appwidget.parent.height == 0) {
            appwidget.scalefactor = 1.0
            console.log("父类窗口大小为 0 默认缩放比为1.0")
            return
        }
        appwidget.scalefactor = appwidget.parent.width/448
        //console.log("当前比例系数", appwidget.scalefactor);
        var tw = appwidget.scalefactor *448
        var th = appwidget.scalefactor *162
        //console.log("大小：", tw,th ,appwidget.height);
        if(th > appwidget.parent.height) {
            appwidget.scalefactor = appwidget.parent.height/162
            //console.log("以宽度计算比例系数", appwidget.scalefactor);
        }
    }
    function onUserStatusChanged(status)
    {
        console.log("qml Appwidget demosearch user status changed ", status);
        if(status === "Normal") {
            appwidget.qmlenable = true;
        }else if(status === "Editable") {
            appwidget.qmlenable = false;
        }
    }
    function updateTimeInfo()
    {
        var timeInfo = "";
        timeInfo = (appwidget.timeformat === "12") ? "hh mm AP" : "hh mm";
        updateSystemTime(timeInfo);
        if (timeInfo === "hh mm AP") {
            updateTimeRecWidth(appwidget.hour);
            ampm.text = Qt.formatTime(new Date(), "AP");
        }
        time1.text = appwidget.hour + ":" + appwidget.minute;
        time2.text = appwidget.hour + ":" + appwidget.minute
    }
    function updateSystemTime(timeString)
    {
        var timeStr = "";
        timeStr = Qt.formatTime(new Date(), timeString).trim().split(/\s+/);
        appwidget.hour = timeStr[0];
        appwidget.minute = timeStr[1];
    }
    function updateTimeRecWidth(hh)
    {
        var hournumber = 0;
        hournumber = Number(hh);
        appwidget.hour = hournumber.toString();
        time1rec.width = hournumber < 10 ? 244*appwidget.scalefactor
                                         : 294*appwidget.scalefactor
    }
    function updateWeakInfo()
    {
        week1.text = Qt.formatDate(new Date(), "ddd")
        week2.text = Qt.formatDate(new Date(), "ddd")
    }
    function updateDateInfo()
    {
        var dateInfo = "";
        if(dateformat == "cn") {
            dateInfo = "MM/dd";
        }
        if(dateformat == "en") {
            dateInfo = "MM-dd";
        }
        date1.text = Qt.formatDate(new Date(), dateInfo);
        date2.text = Qt.formatDate(new Date(), dateInfo);
    }
}



